# Generated by Django 4.2.16 on 2024-10-17 07:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("forum", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="readstate",
            name="course_id",
            field=models.CharField(max_length=255),
        ),
        migrations.AlterUniqueTogether(
            name="lastreadtime",
            unique_together={("read_state", "comment_thread")},
        ),
        migrations.CreateModel(
            name="MongoContent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content_object_id", models.PositiveIntegerField(null=True)),
                ("mongo_id", models.CharField(max_length=50, unique=True)),
                (
                    "content_type",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
    ]
