# Generated by Django 4.2.17 on 2025-03-04 06:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("forum", "0002_alter_readstate_course_id_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="commentthread",
            name="title",
            field=models.CharField(max_length=1024),
        ),
    ]
